@@ -5,6 +5,7 @@
 */
 
 #include "textdocumentprocessor.h"
+#include "genericpriceextractorhelper_p.h"
 
 #include <KItinerary/ExtractorFilter>
 
@@ -33,3 +34,8 @@ bool TextDocumentProcessor::matches(const ExtractorFilter &filter, const Extract
 {
     return filter.matches(node.content<QString>());
 }
+
+void TextDocumentProcessor::postExtract(ExtractorDocumentNode &node, [[maybe_unused]] const ExtractorEngine *engine) const
+{
+    GenericPriceExtractorHelper::postExtract(node.content<QString>(), node);
+}
