@@ -91,6 +91,19 @@ static ProgramMembership extractCustomerCard(const QList <Fcb::TariffType> &tari
     return {};
 }
 
+static void fixFcbStationCode(TrainStation &station)
+{
+    // UIC codes in Germany are wildly unreliable, there seem to be different
+    // code tables in use by different operators, so we unfortunately have to ignore
+    // those entirely
+    if (station.identifier().startsWith(QLatin1String("uic:80"))) {
+        PostalAddress addr;
+        addr.setAddressCountry(QStringLiteral("DE"));
+        station.setAddress(addr);
+        station.setIdentifier(QString());
+    }
+}
+
 void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_unused]] const ExtractorEngine *engine) const
 {
     const auto p = node.content<Uic9183Parser>();
@@ -199,6 +212,7 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
 
     const auto fcb = p.findBlock<Fcb::UicRailTicketData>();
     if (fcb.isValid()) {
+        res.setPriceCurrency(QString::fromUtf8(fcb.issuingDetail.currency));
         const auto issueDt = fcb.issuingDetail.issueingDateTime();
         for (const auto &doc : fcb.transportDocument) {
             if (doc.ticket.userType() == qMetaTypeId<Fcb::ReservationData>()) {
@@ -209,11 +223,13 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                 TrainStation dep;
                 dep.setName(irt.fromStationNameUTF8);
                 dep.setIdentifier(FcbUtil::fromStationIdentifier(irt));
+                fixFcbStationCode(dep);
                 trip.setDepartureStation(dep);
 
                 TrainStation arr;
                 arr.setName(irt.toStationNameUTF8);
                 arr.setIdentifier(FcbUtil::toStationIdentifier(irt));
+                fixFcbStationCode(arr);
                 trip.setArrivalStation(arr);
 
                 trip.setDepartureTime(irt.departureDateTime(issueDt));
@@ -230,7 +246,7 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                 if (irt.placesIsSet()) {
                     s.setSeatSection(QString::fromUtf8(irt.places.coach));
                     QStringList l;
-                    for (auto b : irt.places.placeIA5)
+                    for (const auto &b : irt.places.placeIA5)
                         l.push_back(QString::fromUtf8(b));
                     for (auto i : irt.places.placeNum)
                         l.push_back(QString::number(i));
@@ -242,6 +258,10 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                 t.setTicketedSeat(s);
                 res.setProgramMembershipUsed(extractCustomerCard(irt.tariffs));
 
+                if (irt.priceIsSet()) {
+                    res.setTotalPrice(irt.price / std::pow(10, fcb.issuingDetail.currencyFract));
+                }
+
                 if (validator.isValidElement(trip)) {
                     res.setReservationFor(trip);
                     res.setReservedTicket(t);
@@ -257,6 +277,10 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                 t.setTicketedSeat(s);
                 res.setProgramMembershipUsed(extractCustomerCard(nrt.tariffs));
 
+                if (nrt.priceIsSet()) {
+                    res.setTotalPrice(nrt.price / std::pow(10, fcb.issuingDetail.currencyFract));
+                }
+
                 // check for TrainLinkType regional validity constrains
                 bool trainLinkTypeFound = false;
                 for (const auto &regionalValidity : nrt.validRegion) {
@@ -270,10 +294,12 @@ void Uic9183DocumentProcessor::preExtract(ExtractorDocumentNode &node, [[maybe_u
                     // TODO station identifier
                     TrainStation dep;
                     dep.setName(trainLink.fromStationNameUTF8);
+                    fixFcbStationCode(dep);
                     trip.setDepartureStation(dep);
 
                     TrainStation arr;
                     arr.setName(trainLink.toStationNameUTF8);
+                    fixFcbStationCode(arr);
                     trip.setArrivalStation(arr);
 
                     trip.setDepartureTime(trainLink.departureDateTime(issueDt));
